from PIL import Image, ImageDraw
import numpy as np
import timeit

from spriteutil import find_most_common_color
from sprite_class import Sprite
from create_sprite_labels_image import create_sprite_labels_image
from find_sprite import find_sprites

class SpriteSheet:
    def __init__(self, fd, background=None):
        self.fd = self.__image_obj(fd)
        if background:
            self._background_color = background
        else:
            self._background_color = SpriteSheet.find_most_common_color(self.fd)
        self.__label_map = ''
        self.__sprites = ''
    
    @property
    def background_color(self):
        return self._background_color

    def __image_obj(self, fd):
        try:
            return Image.open(fd)
        except (AttributeError, UnicodeDecodeError):
            try:
                return Image.open(fd.name)
            except Exception:
                return fd

    @staticmethod
    def find_most_common_color(image):
        """
        Get the pixel color that is the most used in this image
        @param image : a Image object
        @return : most common color in image
        """
        return find_most_common_color(image)
    
    
    def find_sprites(self):
        """ 
        Detect sprites inside the image
        Return a 2D label map and a dict that stores:
                key: sprite's label
                value: its Sprite's object
        """
        return find_sprites(self.fd)

    def create_sprite_labels_image(self):
        """
        Draws the masks of the sprites at the exact same position that the sprites were in the original image.
        Returns an image of equal dimension (width and height) as the original image that was passed to the function
        """
        sprites, label_map = self.find_sprites()
        # random color base on mode color (RGBA/RGB)
        return create_sprite_labels_image(sprites, label_map)


def main():
    # image = Image.open('data/Barbarian.gif')
    # img = image.convert('RGB')
    obj = SpriteSheet('data/sprite_sheet_ken_01.png')
    # obj = SpriteSheet(img)
    create_obj = obj.create_sprite_labels_image().save('new.png')
    print(timeit.timeit(stmt=lambda: create_obj, number=1))
    # sprites, label_map = obj.find_sprites()
    # for label, sprite in sprites.items():
    #     s = (f"Sprite ({label}): [{sprite.top_left}, {sprite.bottom_right}] {sprite.width}x{sprite.height}")
    #     print(s)
    # print(timeit.timeit(stmt=lambda: s, number=1))

if __name__ == '__main__':
    main()
